/****************************************************************************
* (C) 1999-2005 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Germany
*
*  http://www.beck-ipc.com
*
* ---------------------------------------------------------------------------
* Module    : FOSSIL.C
* Function  : Fossil interface, accessing the serial devices of the IPC@CHIP
* ---------------------------------------------------------------------------

$Header: FOSSIL.C, 30, 23.03.2006 09:43:41, Andre Pribil$

*****************************************************************************/

#include <dos.h>

#include "config.h"
#include "..\hw\hw_priv.h"
#include "fossil.h"
#include "hwapi.h"

#define FOSSILVECT 0x14

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 Fossil API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces FOSSIL\.H><efont> |
    <tab><linkto>#Configuration<linkmid>Configuration<linktail><tab>
    <linkto>#Transmitter<linkmid>Transmitter<linktail><tab>
    <linkto>#Receiver<linkmid>Receiver<linktail><tab>
 @libcat1  Configuration | fos04   | fossil_init | Initialize the fossil driver |
 @libcat1  Configuration | fos05   | fossil_deinit | Deinitialize fossil driver |
 @libcat1  Configuration | fos0081 | fossil_setbaud | Set baud rate |
 @libcat1  Configuration | fos81   | fossil_set_extctrl | Extended line control initialization |
 @libcat1  Configuration | fos0F   | fossil_set_flowcontrol | Enable/disable flow control |
 @libcat1  Configuration | fos86   | fossil_force_rts | Force RTS signal |
 @libcat1  Configuration | fos87   | fossil_read_cts  | Read CTS signal  |
 @libcat1  Configuration | fosA1   | fossil_user_callback | Install a Fossil User Callback Function |

 @libcat1  Configuration | fos03 | fossil_status_request | <nl>Get port status | <tab> -- Status Functions --<nl>
 @libcat1  Configuration | fos1B | fossil_get_driver_info | Get driver info |
 @libcat1  Configuration | fosA0 | fossil_get_bytes_in_uart |
        Report number of bytes in UART transmit hardware buffers. |

 @libcat1  Configuration | fos80 | fossil_set_rs485 | <nl>Enable/Disable RS485 mode | <tab> -- RS-485 Control --<nl>
 @libcat1  Configuration | fos82 | fossil_set_rs485_txenable | Select RS485 pin |


 @libcat2  Transmitter | fos01 | fossil_putbyte_wait |
        Put byte in output buffer, wait if needed |
 @libcat2  Transmitter | fos0B | fossil_putbyte |
        Put byte in output buffer (without wait) |
 @libcat2  Transmitter | fos19 | fossil_writeblock |
        Write a block of data |
 @libcat2  Transmitter | fos08 | fossil_flush_output |
        Await empty transmit buffer |
 @libcat2  Transmitter | fos09 | fossil_purge_output |
        Purge output buffer |
 @libcat2  Transmitter | fos83 | fossil_send_break |
        Send break |
 @libcat2  Transmitter | fos85 | fossil_enable_transmitter |
        Enable/Disable transmitter |


 @libcat3  Receiver | fos02 | fossil_getbyte_wait |
        Get byte from input buffer (with wait) |
 @libcat3  Receiver | fos18A | fossil_getbyte |
        Get byte from input buffer (no wait) |
 @libcat3  Receiver | fos0C | fossil_peek_input |
        Peek if next byte is available |
 @libcat3  Receiver | fos18 | fossil_readblock |
        Read block of data |
 @libcat3  Receiver | fos0A | fossil_purge_input |
        Purge input buffer |
 @libcat3  Receiver | fos84 | fossil_enable_receiver |
        Enable/Disable receiver |

*/
/******************************************************************************/
#ifdef _fos0081_
/*
 @doc DOC_fos0081_
 ---------------------------------------------------------------
 Set baudrate
 ---------------------------------------------------------------
 @clib  fossil_setbaud | Set baud rate and port configuration. |
        <cfont>int fossil_setbaud (int port, long baudrate, int parity,<nl>
        <tab><tab><tab><tab><tab>int wordlen, int stopbits );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  int  | baudrate  | Desired baud rate, bits per second
 @parm  int  | parity  | Enumeration type:<ul>
        FOSSIL_PARITY_NO     = 0<nl>
        FOSSIL_PARITY_ODD    = 1<nl>
        FOSSIL_PARITY_EVEN   = 2<nl>
        FOSSIL_PARITY_MARK   = 3<nl>
        FOSSIL_PARITY_SPACE  = 4<ule>
 @parm  int  | wordlen  | Bits per character (7 or 8)
 @parm  int  | stopbits  | Stop bits (1 or 2)
 @rdesc -1 = Illegal parameter<nl>
        Else status code (bit field):<ul>
          bit 6: FOSSIL_OUTPUT_BUFFER_EMPTY<nl>
          bit 5: FOSSIL_OUTPUT_NOT_FULL<nl>
          bit 4: FOSSIL_LINE_BREAK   Line break detected<nl>
          bit 3: FOSSIL_FRAMING_ERROR   Framing error detected<nl>
          bit 2: FOSSIL_PARITY_ERROR   Parity error detected<nl>
          bit 1: FOSSIL_OVERRUN_ERROR   Set if overrun occurred on receiver.<nl>
          bit 0: FOSSIL_DATA_AVAILABLE   Set if data is available in receiver buffer.<ule>
          <nl>
       The header file contains the indicated mask definitions.
 @comm Set baud rate and other parameters for specified port.<nl>
       <nl>
       For standard baud rates, this function uses Fossil interrupt
       0x14 Service <linkto>../../fossil.htm#0x140x00<linkmid>0x00<linktail>.<nl>
       <nl>
       For non-standard baud rates the HAL interrupt 0xA1 "Get Frequencies"
       function <linkto>../../hardware.htm#0xA10x8A<linkmid>0x8A<linktail>
       is used to determine the maximum baud rate.<s2> Based on this
       maximum baud rate a baud rate divider is computed for the
       "Extended Line Control Initialization" Fossil interrupt
       <linkto>../../fossil.htm#0x140x81<linkmid>0x81<linktail> service.<s2>
       Then the Fossil interrupt
       <linkto>../../fossil.htm#0x140x03<linkmid>0x03<linktail> service reports
       the port's status.<nl>
       <nl>
       If the DMA mode (send or receive mode) is enabled at
        <linkto>../../config.htm#SERIAL_SEND_DMA<linkmid>Chip.ini<linktail>,
        the following port settings are not allowed:<ul>
            1. 8N2 (this limitation applies only to SC1x systems)<nl>
            2. 8S2<nl>
            3. 8M2<ule>
               <nl>
 @commtarget SC1x  |
             Two stop bits are only available if no parity is set.<nl>
 @commtarget SC1x3 |
             Two stop bits are only available if no parity is set or word length is
             equal to 7 bits.<nl>

 @Also fos05.htm |  fossil_deinit |
       fos81.htm |  fossil_set_extctrl  |
       fos0F.htm |  fossil_set_flowcontrol
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x81<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int fossil_setbaud (int port,
                    long baudrate,
                    FOSSIL_PARITY  parity,
                    int wordlen,
                    int stopbits)
{
    #define BAUD_LIST_SIZE (sizeof(baudtab)/sizeof(baudtab[0]))

    static unsigned int baudtab[8] = {19200, 38400, 300, 600, 1200, 2400, 4800, 9600};
    int baud_idx;
    long baud_max, baud_div;

    stopbits-- ;
    if (  (wordlen < 7) || (wordlen > 8)
        ||((unsigned int)parity > (unsigned int)FOSSIL_PARITY_SPACE)
        ||((unsigned int)stopbits >= 2)
       )
    {
       return -1 ;                  // Illegal parameters
    }
    wordlen -= 5 ;

    // check parameters
    for (baud_idx = 0; baud_idx < BAUD_LIST_SIZE; baud_idx++)
    {
        if (baudrate == (long) baudtab[baud_idx])
        {
            break;
        }
    }

    // init port
    if (  (baud_idx < BAUD_LIST_SIZE)           // Found in list?
        &&(parity <= FOSSIL_PARITY_EVEN)        // No EXT CTRL needed?
       )
    {
        // use function 0x00 for baudrates up to 38400

        if (parity == FOSSIL_PARITY_EVEN)
        {
            parity = 3 ;
        }

        _AX = (wordlen + (stopbits << 2) + (parity << 3) + (baud_idx << 5))
             & 0xFF ;   // AH = 0 -> SWI service index

    } // if (baud_idx<=7)
    else
    {
        // use extended line control init for other baud rates
        if (parity > 4)
        {
            return -1 ;                 // Illegal parameter
        }

        // get max. baudrate value
       _AX = ((unsigned)HAL_GET_FREQUENCIES << 8)  // AH = service index
           | GET_FRQ_MAX_BAUD ;                    // AL = frequency selector

        asm { int HAL_INT  } ;
        asm { mov WORD PTR baud_max,   AX } ;
        asm { mov WORD PTR baud_max+2, dx } ;


        // calc divider with integer rounding
        baud_div = baud_max*10L / baudrate;
        if ((baud_div % 10L) < 5L)
        {
            baud_div = baud_max / baudrate;
        }
        else
        {
            baud_div = baud_max / baudrate + 1;
        }

        _BH = parity ;
        _BL = stopbits ;
        _DX = port;
        _CX = (unsigned int) baud_div;
        _AL = wordlen ;
        _AH = 0x81;              // Extended line control initialization
        asm { int FOSSILVECT  } ;

        // get status of port for final return value.
        _AH = 0x03 ;

    } // else (baud_idx<=7)

    _DX = port;
    asm { int FOSSILVECT  } ;
    return _AH ;
}  // fossil_setbaud()
#endif

/******************************************************************************/
#ifdef _fos01_
/*
 @doc DOC_fos01_
 ---------------------------------------------------------------
 Put byte in tx buffer, wait for room if needed
 ---------------------------------------------------------------
 @clib  fossil_putbyte_wait | Put byte in output buffer, wait if needed. |
        <cfont>int fossil_putbyte_wait ( int port, int ch );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  int  | ch  | Byte to output
 @rdesc Status  code (bit field):<ul>
          bit 6: FOSSIL_OUTPUT_BUFFER_EMPTY<nl>
          bit 5: FOSSIL_OUTPUT_NOT_FULL<nl>
          bit 4: FOSSIL_LINE_BREAK   Line break detected<nl>
          bit 3: FOSSIL_FRAMING_ERROR   Framing error detected<nl>
          bit 2: FOSSIL_PARITY_ERROR   Parity error detected<nl>
          bit 1: FOSSIL_OVERRUN_ERROR   Set if overrun occurred on receiver.<nl>
          bit 0: FOSSIL_DATA_AVAILABLE   Set if data is available in receiver buffer.<ule>
          <nl>
        The header file contains the indicated mask definitions.
 @comm  Character is queued for transmission.<s2>If there is space
        in the transmitter buffer when this call is made, the character
        will be stored and control returned to caller.<s2> If the buffer
        is full, the driver will wait for space.<s2> (This can be dangerous
        when used in combination with flow control.)
 @Also  fos0B.htm |  fossil_putbyte |
        fos02.htm |  fossil_getbyte_wait |
        fos19.htm |  fossil_writeblock
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x01<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/

int fossil_putbyte_wait(int port, int ch)
{
    _AH = 0x01;
    _AL = (char) ch;
    _DX = port;
    asm { int FOSSILVECT  } ;
    return _AH ;
}
#endif


/******************************************************************************/
#ifdef _fos02_
/*
 @doc DOC_fos02_
 ---------------------------------------------------------------
 get char, wait if needed
 ---------------------------------------------------------------
 @clib  fossil_getbyte_wait | Get byte from input buffer (with wait). |
        <cfont>int fossil_getbyte_wait ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc Received byte
 @comm  Reads a byte from the receiver buffer.<s2> Wait for a byte
        to arrive if none is available.
 @Also  fos18A.htm |  fossil_getbyte |
        fos18.htm |  fossil_readblock |
        fos01.htm |  fossil_putbyte_wait
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x02<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int fossil_getbyte_wait (int port)
{
    _AH = 0x02;
    _DX = port;
    asm { int FOSSILVECT  } ;
    return _AL ;
}
#endif


/******************************************************************************/
#ifdef _fos03_
/*
 @doc DOC_fos03_
 ---------------------------------------------------------------
 Get port status
 ---------------------------------------------------------------
 @clib  fossil_status_request | Get port status. |
        <cfont>int fossil_status_request ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc Status  code (bit field):<ul>
          bit 6: FOSSIL_OUTPUT_BUFFER_EMPTY<nl>
          bit 5: FOSSIL_OUTPUT_NOT_FULL<nl>
          bit 4: FOSSIL_LINE_BREAK   Line break detected<nl>
          bit 3: FOSSIL_FRAMING_ERROR   Framing error detected<nl>
          bit 2: FOSSIL_PARITY_ERROR   Parity error detected<nl>
          bit 1: FOSSIL_OVERRUN_ERROR   Set if overrun occurred on receiver.<nl>
          bit 0: FOSSIL_DATA_AVAILABLE   Set if data is available in receiver buffer.<ule>
          <nl>
        The header file contains the indicated mask definitions.
 @comm  Any reported UART error flags are cleared after the
        read is made for this call.<nl>
        <nl>
        For efficient receiver polling, the
        <linkto>fos18A.htm<linkmid>fossil_getbyte<linktail> or
        <linkto>fos18.htm<linkmid>fossil_readblock<linktail> methods
        are recommended.
 @Also  fos02.htm  | fossil_getbyte_wait |
        fos18A.htm | fossil_getbyte |
        fos18.htm  | fossil_readblock
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x03<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int fossil_status_request (int port)
{
    _AH = 0x03;
    _DX = port;
    asm { int FOSSILVECT  } ;
    return _AH ;
}
#endif

/*****************************************************************************
*/
#ifdef _fos04_
/*
 @doc DOC_fos04_
 ---------------------------------------------------------------
 Initialize fossil driver
 ---------------------------------------------------------------
 @clib  fossil_init | Initialize the fossil driver for specified port |
        <cfont>int fossil_init ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc 0x1954, signature from Fossil driver
 @comm  Use this function to initialize the fossil interface for a specified port.<s2>
        The user must make sure that only one process opens a port.<s2>
        If this port is used for standard
        <linkto>../../config.htm#STDIO_STDIN<linkmid>input<linktail>
        or <linkto>../../config.htm#STDIO_STDOUT<linkmid>output<linktail>
        (console), then stdin/stdout will be disabled for this port.
 @Also  fos05.htm   |  fossil_deinit |
        fos0081.htm | fossil_setbaud |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x04<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int fossil_init (int port)
{
    _DX = port ;
    _AH = 0x04 ;                // Service index
    asm { int FOSSILVECT  } ;
    return _AX ;
}
#endif


/******************************************************************************/
#ifdef _fos05_
/*
 @doc DOC_fos05_
 ---------------------------------------------------------------
 Deinitialize fossil driver
 ---------------------------------------------------------------
 @clib  fossil_deinit | Deinitialize fossil driver. |
        <cfont>void fossil_deinit ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc -- none --
 @comm  This function restores the specified RS-232 port to
        its default settings.
 @Also  fos04.htm   | fossil_init |
        fos0081.htm | fossil_setbaud  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x05<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void fossil_deinit(int port)
{
    _DX = port ;
    _AH = 0x05 ;                // Service index
    asm { int FOSSILVECT  } ;
}
#endif

/******************************************************************************/
#ifdef _fos08_
/*
 @doc DOC_fos08_
 ---------------------------------------------------------------
 Flush output buffer, wait until done
 ---------------------------------------------------------------
 @clib  fossil_flush_output | Flush output buffer, wait until done. |
        <cfont>void fossil_flush_output ( int port );<efont>  |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc -- none --
 @comm  Wait for all output in the output buffer to be transmitted.
 @Also  fos09.htm |  fossil_purge_output |
        fos0A.htm |  fossil_purge_input  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x08<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void fossil_flush_output (int port)
{
    _DX = port ;
    _AH = 0x08 ;                // Service index
    asm { int FOSSILVECT  } ;
}
#endif


/******************************************************************************/
#ifdef _fos09_
/*
 @doc DOC_fos09_
 ---------------------------------------------------------------
 Purge output buffer
 ---------------------------------------------------------------
 @clib  fossil_purge_output | Purge output buffer and discard the data. |
        <cfont>void fossil_purge_output ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc -- none --
 @comm  Remove all data from the output buffer and discard it.
 @Also  fos08.htm |  fossil_flush_output |
        fos0A.htm |  fossil_purge_input  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x09<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void fossil_purge_output (int port)
{
    _DX = port ;
    _AH = 0x09 ;                // Service index
    asm { int FOSSILVECT  } ;
}
#endif


/******************************************************************************/
#ifdef _fos0A_
/*
 @doc DOC_fos0A_
 ---------------------------------------------------------------
 Purge input buffer
 ---------------------------------------------------------------
 @clib  fossil_purge_input | Purge input buffer and discard the data. |
        <cfont>void fossil_purge_input ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc -- none --
 @comm  Remove all data from the receive buffer and discard the data.
 @Also  fos08.htm |  fossil_flush_output |
        fos09.htm |  fossil_purge_output | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x0A<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void fossil_purge_input (int port)
{
    _DX = port ;
    _AH = 0x0A ;                // Service index
    asm { int FOSSILVECT  } ;
}  // fossil_purge_input()
#endif


/******************************************************************************/
#ifdef _fos0B_
/*
 @doc DOC_fos0B_
 ---------------------------------------------------------------
 Put byte in output buffer (without wait)
 ---------------------------------------------------------------
 @clib  fossil_putbyte | Put byte in output buffer (without wait). |
        <cfont>int fossil_putbyte ( int port, int ch );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  int  | ch  | Byte to send
 @rdesc 0 if byte was not accepted (no space in buffer)<nl>
        1 if byte was placed in output buffer
 @comm  Place a byte into the transmit buffer if there is space available.<s2>
        Otherwise simply return with 0, without handling the transmit byte.
 @Also  fos01.htm  | fossil_putbyte_wait |
        fos19.htm  | fossil_writeblock   |
        fos18A.htm | fossil_getbyte
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x0B<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int fossil_putbyte (int port, int ch)
{
    _DX = port ;
    _AL = (char) ch ;
    _AH = 0x0B ;                // Service index
    asm { int FOSSILVECT  } ;
    return _AX ;
}
#endif

/******************************************************************************/
#ifdef _fos0C_
/*
 @doc DOC_fos0C_
 ---------------------------------------------------------------
 Peek if next byte is available
 ---------------------------------------------------------------
 @clib  fossil_peek_input | Peek if next byte is available. |
        <cfont>int fossil_peek_input ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc -1 if no character available,<nl>
        Else next character in receiver buffer.
 @comm  Returns the next byte available in the receive buffer,
        without removing it from the buffer.
 @Also  fos02.htm  | fossil_getbyte_wait |
        fos18A.htm | fossil_getbyte  |
        fos03.htm  | fossil_status_request
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x0C<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int fossil_peek_input(int port)
{
    _DX = port ;
    _AH = 0x0C ;                // Service index
    asm { int FOSSILVECT  } ;
    return (int)_AX;
}
#endif


/******************************************************************************/
#ifdef _fos0F_
/*
 @doc DOC_fos0F_
 ---------------------------------------------------------------
 Enable/disable flow control
 ---------------------------------------------------------------
 @clib  fossil_set_flowcontrol | Enable/disable flow control. |
        <cfont>void fossil_set_flowcontrol ( int port, FOS_FLOW_CTRL flowctrl );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  int  | flowctrl  | Enumerator specifies desired type of
        flow control:<nl>
        <nl>
        FOSSIL_FLOWCTRL_OFF = 0x0: No flow control<nl>
        FOSSIL_FLOWCTRL_XONXOFF_SEND = 0x1:<nl><ul>
          XON/XOFF on transmit<s2>(watch for XOFF while sending)
          <ule>
        FOSSIL_FLOWCTRL_RTSCTS       = 0x2:<nl><ul>
            CTS/RTS (CTS on transmit/RTS on receive)
            <ule>
        FOSSIL_FLOWCTRL_XONXOFF_RECV = 0x8:<nl><ul>
            XON/XOFF on receive (send XOFF when buffer near full)
            <ule>
        FOSSIL_FLOWCTRL_XONXOFF_SEND_RECV = 0x9:<nl><ul>
            XON/XOFF used on both Tx and Rx
        <ule>
 @rdesc  -- none --
 @comm  Configures the flow control for a port.<nl>
        <nl>
        Since SC12 @CHIP-RTOS 1.02B XON/XOFF mode is also available if the
        <linkto>../../config.htm#SERIAL_EXT_DMA<linkmid>DMA<linktail>
        mode for the COM port is enabled but because of the internal
        functionality of DMA it is not possible to detect an XON or XOFF
        of the peer immediateley.<s2> It is possible that an overrun situation
        at the connected peer (e.g. GSM modem) could occur.<s2> We enable this
        mode now because GSM modems (any??) supports only XON/XOFF flow ctrl.<nl>
 @Also  fos0081.htm |  fossil_setbaud | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x0F<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void fossil_set_flowcontrol (int port, FOS_FLOW_CTRL flowctrl)
{
    _DX = port ;
    _AL = (unsigned char) flowctrl ;
    _AH = 0x0F ;                // Service index
    asm { int FOSSILVECT  } ;
}
#endif


/******************************************************************************/
#ifdef _fos18_
/*
 @doc DOC_fos18_
 ---------------------------------------------------------------
 Read block of data
 ---------------------------------------------------------------
 @clib  fossil_readblock | Read block of data (no wait). |
        <cfont>unsigned int fossil_readblock ( int port,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab>unsigned char far *buffer,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab>unsigned int count );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  int  | buffer  | Output parameter:<s2>Pointer to
        buffer to put received bytes into.
 @parm  int  | count  | Size of <code>buffer<codee>, maximum
        number of bytes to transfer.
 @rdesc Number of bytes transfered
 @comm  Read up to a specified number of bytes from a serial port
        input <linkto>../../CONFIG.HTM#SERIAL_EXT_RECVQUEUE<linkmid>queue<linktail>
        and return immediately.
 @Also  fos19.htm  | fossil_writeblock |
        fos02.htm  | fossil_getbyte_wait |
        fos18A.htm | fossil_getbyte
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x18<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
unsigned int fossil_readblock (int port,
                               unsigned char far *buffer,
                               unsigned int count)
{
    asm { LES  DI, buffer } ;
    _CX = count ;
    _DX = port ;
    _AH = 0x18 ;                // Service index
    asm { int FOSSILVECT  } ;
    return _AX ;
}
#endif

/******************************************************************************/
#ifdef _fos18A_
/*
 @doc DOC_fos18A_
 ---------------------------------------------------------------
 Read block of data
 ---------------------------------------------------------------
 @clib  fossil_getbyte | Get byte from input buffer (no wait). |
        <cfont>int fossil_getbyte ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc -1 if no data is available in receiver buffer,
        else next received byte
 @comm  This function uses Fossil interrupt
        0x14 Service <linkto>../../fossil.htm#0x140x18<linkmid>0x18<linktail>
        to read out a block of length one.<s2>This method of polling for
        single characters at the receiver will be considerably more
        efficient than the alternative method using the
        <linkto>fos03.htm<linkmid>fossil_status_request<linktail>
        and <linkto>fos02.htm<linkmid>fossil_getbyte_wait<linktail>
        function pair.
 @Also  fos18.htm | fossil_readblock |
        fos02.htm | fossil_getbyte_wait |
        fos03.htm | fossil_status_request
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x18<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int fossil_getbyte (int port)
{
    unsigned int buffer ;      // Read out one char into here.
    unsigned int read_count ;

    buffer = 0 ;                // Clear upper byte.

    _ES = _SS ;
    asm { LEA  DI, buffer } ;   // ES:DI = far ptr to buffer
    _CX = 1 ;
    _DX = port ;
    _AH = 0x18 ;                // Service index
    asm { int FOSSILVECT  } ;
    read_count = _AX ;
    if (read_count == 0)        // No data ready?
    {
        buffer = -1 ;           // Set "no data" indication
    }
    return buffer ;
}
#endif

/******************************************************************************/
#ifdef _fos19_
/*
 @doc DOC_fos19_
 ---------------------------------------------------------------
 Write a block of data
 ---------------------------------------------------------------
 @clib  fossil_writeblock | Write a block of data (no wait). |
        <cfont>unsigned int fossil_writeblock ( int port,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab> const unsigned char far *buffer,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab> unsigned int count );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  int  | buffer  | Input parameter:<s2>Pointer to
        data to be transmitted
 @parm  int  | count  | Maximum number of bytes to transfer.
 @rdesc Number of bytes transfered to transmit buffer from
        <code>buffer<codee>.
 @comm  Write a block of data to the serial port output buffer.<s2>
        This function returns immediately after transfering as many
        bytes as <linkto>../../CONFIG.HTM#SERIAL_EXT_SENDQUEUE<linkmid>space<linktail>
        in transmit buffer allows.
 @Also  fos18.htm |  fossil_readblock |
        fos01.htm |  fossil_putbyte_wait |
        fos0B.htm |  fossil_putbyte
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x19<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
unsigned int fossil_writeblock (int port,
                                const unsigned char far *buffer,
                                unsigned int count)
{
    asm { LES  DI, buffer } ;
    _CX = count ;
    _DX = port ;
    _AH = 0x19 ;                // Service index
    asm { int FOSSILVECT  } ;
    return _AX ;
}
#endif

/******************************************************************************/

/***************************************************************************/
//  FossilDriverInfo Data Structure
/***************************************************************************/
/*
 @doc DOC_FosDrInfo_
 ---------------------------------------------------------------
 FossilDriverInfo
 ---------------------------------------------------------------
 @cstruc FossilDriverInfo data structure | Fossil Driver Info |
         <cfont>
         typedef<s2>struct FossilDriverInfo<nl>
         {<nl>
         <tab>unsigned size;<nl>
         <tab>unsigned char version;<s2>// (not used)<nl>
         <tab>unsigned char level;<s2>// (not used)<nl>
         <tab>char far *id;<s2>// (not used)<nl>
         <nl>
         <tab>unsigned inputBuffSize;<nl>
         <tab>unsigned inputAvailable;<nl>
         <tab>unsigned outputBuffSize;<nl>
         <tab>unsigned outputAvailable;<nl>
         <tab>unsigned char screenWidth;<s2>// (not used)<nl>
         <tab>unsigned char screenHeight;<s2>// (not used)<nl>
         <tab>unsigned char baudrateMask;<s2>// (not used)<nl>
         }  FossilDriverInfo_t ; | |
 @element   int  | size  | Size of this data structure, set to 19 by
            <linkto>fos1B.htm<linkmid>fossil_get_driver_info<linktail>.
 @element   int  | inputBuffSize  | Size of receiver buffer.
 @element   int  | inputAvailable  | Bytes available in receiver buffer.
 @element   int  | outputBuffSize  | Size of transmit buffer.
 @element   int  | outputAvailable  | Space available in transmit buffer.
 @comm   Unused fields are set to zero by
         <linkto>fos1B.htm<linkmid>fossil_get_driver_info<linktail>.
 @Also  fos1B.htm |  fossil_get_driver_info | | | |
*/
/******************************************************************************/
#ifdef _fos1B_
/*
 @doc DOC_fos1B_
 ---------------------------------------------------------------
 Get driver info
 ---------------------------------------------------------------
 @clib  fossil_get_driver_info | Get driver info. |
        <cfont>int fossil_get_driver_info (int port, FossilDriverInfo_t far *drv_info);<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  void * | drv_info  | Output parameter:<s2>  Pointer to
        <linkto>FosInfo.htm<linkmid>FossilDriverInfo<linktail>
        data structure which will be filled by this call.
 @rdesc 13, = sizeof(FossilDriverInfo_t)
 @comm  Get information about a serial port and driver.
 @Also  fosA0.htm |  fossil_get_bytes_in_uart | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x1B<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int fossil_get_driver_info (int port, FossilDriverInfo_t far *drv_info)
{
    asm { LES  DI, drv_info } ;
    _CX = sizeof (FossilDriverInfo_t) ;
    _DX = port ;
    _AH = 0x1B ;                // Service index
    asm { int FOSSILVECT  } ;
    return _AX ;
}
#endif

/******************************************************************************/
#ifdef _fos80_
/*
 @doc DOC_fos80_
 ---------------------------------------------------------------
 Enable/Disable RS485 mode
 ---------------------------------------------------------------
 @clib  fossil_set_rs485 | Enable/Disable RS485 mode. |
        <cfont>void fossil_set_rs485 ( int port, RS485_MODE mode);<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  int  | mode  | RS 485 mode enumerator:<nl><ul>
        FOSSIL_RS485_LOWACTIVE = 0:<s2> TxEnable low active<nl>
        FOSSIL_RS485_HIGHACTIVE = 1:<s2> TxEnable high active<nl>
        FOSSIL_RS485_DISABLE<tab>&nbsp; = 2:<s2> Disable RS485 mode<ule>
 @rdesc -- none --
 @comm  By default the RTS signals (pins) are used to
        enable/disable the respective transmitter. (TxEnable)
 @Also  fos0F.htm |  fossil_set_flowcontrol |
        fos82.htm |  fossil_set_rs485_txenable | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x80<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void fossil_set_rs485(int port, RS485_MODE mode)
{
    _DX = port ;
    _AL = mode ;
    _AH = 0x80 ;                // Service index
    asm { int FOSSILVECT  } ;
}
#endif

/******************************************************************************/
#ifdef _fos81_
/*
 @doc DOC_fos81_
 ---------------------------------------------------------------
 Extended line control initialization
 ---------------------------------------------------------------
 @clib  fossil_set_extctrl | Extended line control initialization. |
        <cfont>void fossil_set_extctrl ( int port, int baud_divider,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; FOSSIL_PARITY parity, int wordlen,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; int stopbits );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  int  | baud_divider  | Baud rate divider
        (For maximum baud rate see
        <linkto>../HW/hal8A.htm<linkmid>hal_get_frequencies<linktail> function.)
 @parm  int  | parity  | Enumeration type:<ul>
        FOSSIL_PARITY_NO     = 0<nl>
        FOSSIL_PARITY_ODD    = 1<nl>
        FOSSIL_PARITY_EVEN   = 2<nl>
        FOSSIL_PARITY_MARK   = 3<nl>
        FOSSIL_PARITY_SPACE  = 4<ule>
 @parm  int  | wordlen  | Bits per character (7 or 8)
 @parm  int  | stopbits  | Stop bits (1 or 2)
 @rdesc  -- none --
 @comm  If the DMA mode (send or receive mode) is enabled at
        <linkto>../../config.htm#SERIAL_SEND_DMA<linkmid>Chip.ini<linktail>,
        the following port settings are not allowed:<ul>
            1. 8N2 (this limitation applies only to SC1x systems)<nl>
            2. 8S2<nl>
            3. 8M2<ule>
 @commtarget SC1x  |
             Two stop bits are only available if no parity is set.<nl>
 @commtarget SC1x3 |
             Two stop bits are only available if no parity is set or word length is
             equal to 7 bits.<nl>
 @Also  fos0081.htm |  fossil_setbaud | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x81<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void fossil_set_extctrl (int port, int baud_divider, FOSSIL_PARITY parity,
                         int wordlen, int stopbits)
{
    _BH = parity ;
    _BL = stopbits - 1 ;
    _DX = port;
    _CX = baud_divider ;
    _AL = wordlen - 5 ;
    _AH = 0x81;               // Extended line control initialization
    asm { int FOSSILVECT  } ;
}
#endif


/******************************************************************************/
#ifdef _fos82_
/*
 @doc DOC_fos82_
 ---------------------------------------------------------------
 Select RS485 pin
 ---------------------------------------------------------------
 @clib  fossil_set_rs485_txenable | Select RS485 pin. |
        <cfont>void fossil_set_rs485_txenable ( int port, int pin );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x  int | pin  | PIO pin number, [0..13]
 @parm_sc1x3 int | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm_sc1x3 int | pin  | PIO pin number, [0..24,26..31]<nl>
             Use -1 to select the RTS pin (hardware functionality / default)
 @rdesc -- none --
 @comm  By default the RTS signals (pins) are used to enable/disable the
        respective transmitter. (TxEnable)<s2>
        This function lets you select any PIO as TxEnable (but not all
        make sense).<s2>To change the default, call this function before you call
        the <linkto>fos80.htm<linkmid>fossil_set_rs485<linktail> function.<s2>
 @Also  fos80.htm |  fossil_set_rs485 | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x82<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void fossil_set_rs485_txenable (int port, int pin)
{
    _DX = port;
    _AL = pin ;
    _AH = 0x82;                       // Service index
    asm { int FOSSILVECT  } ;
}
#endif


/******************************************************************************/
#ifdef _fos83_
/*
 @doc DOC_fos83_
 ---------------------------------------------------------------
 Send break.
 ---------------------------------------------------------------
 @clib  fossil_send_break | Send break |
        <cfont>void fossil_send_break ( int port, FOS_BREAK_SIZE break_size );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  int  | break_size  | Enumeration type:<ul>
        FOS_LONG_BREAK = 1, long break<nl>
        FOS_SHORT_BREAK = 2, short break<nl>
        FOS_EXTRALONG_BREAK = 3, extra long break<ule>
 @rdesc -- none --
 @comm  A short break is a continuous Low on the TXD output for a duration of
        more than one frame transmission time M, where:<nl>
        <nl>
        <tab>M = startbit + data bits (+ parity bit)+ stop bit<nl>
        <nl>
        A long break is a continuous Low on the TXD output for a duration
        of more than two frame transmission times plus the transmission
        time for three additional bits (2M+3).<nl>
        <nl>
        A extra long break is a continuous Low on the TXD output for a duration
        of more than three frame transmission times.
 @Also  fos0081.htm |  fossil_setbaud | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x83<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
void fossil_send_break (int port, FOS_BREAK_SIZE break_size )
{
    _DX = port;
    _AL = (unsigned char) break_size ;
    _AH = 0x83;                       // Service index
    asm { int FOSSILVECT  } ;
}
#endif


/******************************************************************************/
#ifdef _fos84_
/*
 @doc DOC_fos84_
 ---------------------------------------------------------------
 Enable/Disable receiver.
 ---------------------------------------------------------------
 @clib  fossil_enable_receiver | Enable/Disable receiver |
        <cfont>void fossil_enable_receiver ( int port, unsigned char mode );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  unsigned char  | mode  | Mode:<s2> FOS_DISABLE = 0, disable receiver,<s2>
                                           FOS_ENABLE = 1, enable receiver<ule>
 @rdesc -- none --
 @comm This function is useful when using the SM35 as RS485 adapter.
       Use this function to disable the receiver before transmitting data.
       Wait until all data is send and reenable the receiver again.
       This way you don't receive your own transmitted data.<nl><nl>

       By default the receiver is enabled.
 @Also fos85.htm |  fossil_enable_transmitter | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x84<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.10 | V1.00 | V1.00 | V0.90
*/
void fossil_enable_receiver (int port, unsigned char mode)
{
    _DX = port;
    _AL = mode ;
    _AH = 0x84;                       // Service index
    asm { int FOSSILVECT  } ;
}
#endif

/******************************************************************************/
#ifdef _fos85_
/*
 @doc DOC_fos85_
 ---------------------------------------------------------------
 Enable/Disable transmitter.
 ---------------------------------------------------------------
 @clib  fossil_enable_transmitter | Enable/Disable transmitter |
        <cfont>void fossil_enable_transmitter ( int port, unsigned char mode );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  unsigned char  | mode  | Mode:<s2> FOS_DISABLE = 0, disable transmitter,<s2>
                                           FOS_ENABLE = 1, enable transmitter<ule>
 @rdesc -- none --
 @comm  By default the receiver is enabled.
 @Also  fos84.htm |  fossil_enable_receiver | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x85<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.10 | V1.00 | V1.00 | V0.90
*/
void fossil_enable_transmitter (int port, unsigned char mode)
{
    _DX = port;
    _AL = mode ;
    _AH = 0x85;                       // Service index
    asm { int FOSSILVECT  } ;
}
#endif


/******************************************************************************/
#ifdef _fos86_
/*
 @doc DOC_fos86_
 ---------------------------------------------------------------
 Force RTS
 ---------------------------------------------------------------
 @clib  fossil_force_rts | Force RTS# to low, high or reset to normal activity |
        <cfont>void fossil_force_rts ( int port, unsigned char mode );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  unsigned char  | mode  | Mode:<nl><ul>
                RTS_LOW = 0, force RTS to low<nl>
                RTS_HIGH = 1, force RTS to high<nl>
                RTS_NORMAL = 2, reset to normal activity<ule>
 @rdesc -- none --
 @comm  RTS/CTS flow control must be enabled to use this function.
 @Also  fos0F.htm | fossil_set_flowcontrol | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x86<linkmid>documentation<linktail>
        for more details.
 @target      SC12  | SC13  | SC11  | SC1x3
 @version     V1.20 | V1.20 | V1.20 | V1.05

 @target_clib V2.04
*/
void fossil_force_rts (int port, unsigned char mode)
{
    _DX = port;
    _AL = mode ;
    _AH = 0x86;                       // Service index
    asm { int FOSSILVECT  } ;
}
#endif


/******************************************************************************/
#ifdef _fos87_
/*
 @doc DOC_fos87_
 ---------------------------------------------------------------
 Read CTS
 ---------------------------------------------------------------
 @clib  fossil_read_cts | Read level on CTS# pin |
        <cfont>unsigned char fossil_read_cts ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc 0, CTS# is low<nl>
        1, CTS# is high<nl>
 @comm  RTS/CTS flow control must be enabled to use this function.
 @Also  fos0F.htm | fossil_set_flowcontrol | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140x87<linkmid>documentation<linktail>
        for more details.
 @target      SC12  | SC13  | SC11  | SC1x3
 @version     V1.20 | V1.20 | V1.20 | V1.05

 @target_clib V2.04
*/
unsigned char fossil_read_cts (int port)
{
    _DX = port;
    _AH = 0x87;                       // Service index
    asm { int FOSSILVECT  } ;
    return _AL;
}
#endif


/******************************************************************************/
#ifdef _fosA0_
/*
 @doc DOC_fosA0_
 ---------------------------------------------------------------
 Get Bytes in UART
 ---------------------------------------------------------------
 @clib  fossil_get_bytes_in_uart | Get number of bytes currently
        in UART hardware transmit buffers. |
        <cfont>int fossil_get_bytes_in_uart ( int port );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @rdesc Number of transmitter bytes in the UART, pending output<nl>
        -1 if UART transmitter is disabled (since SC1x version V1.20 and SC1x3 version V1.05)
 @comm  Returns the number of bytes which are currently in the UART
        hardware transmit buffers.<nl>
        <nl>
        With this function you can check how many bytes are currently stored at the
        internal UART transmit registers.<s2> This may be necessary to know if your communication pauses
        because of a handshake problem.<s2> If you want to know, how many bytes
        are stored at the fossil driver software transmit queue, you can use
        <linkto>fos1B.htm<linkmid>fossil_get_driver_info<linktail> function.<s2>
        However, fossil_get_driver_info does not check the UART transmit
        hardware buffers.
 @Also  fos1B.htm |  fossil_get_driver_info | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140xA0<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90
*/
int fossil_get_bytes_in_uart (int port)
{
    _DX = port;
    _AH = 0xA0 ;                      // Service index
    asm { int FOSSILVECT  } ;
    return _AX ;
}
#endif

/******************************************************************************/

/***************************************************************************/
//  Fossil User CallBack Event Data Structure
/***************************************************************************/
/*
 @doc DOC_FosUCBEvent_
 ---------------------------------------------------------------
 FossilUCBEvent
 ---------------------------------------------------------------
 @cstruc  Fossil User CallBack Event data structure | Fossil User CallBack Event |
        <cfont>
        typedef struct<nl>
        {<nl>
        <tab>int size;<s2><nl>
        <tab>int port;<s2><nl>
        <tab>int event;<s2><nl>
        <tab>void far *data;<s2><nl>
        <tab>unsigned data_length;<s2><nl>
        }  fossil_event_t ; | |
 @element int  | size  | Size of this struct (sizeof(fossil_event_t))
 @element int  | port  | Serial port:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port (SC1x3 only)<nl>
                FOSSIL_SER3 = 3 for SER3 port (SC1x3 only)<nl>
                <ule>
 @element int  | event  | Possible events<nl>
 @element void*  | data  | Data pointer
 @element unsigned  | data_length  | Data length
 @comm  The fossil_event_struct contains the event which has occured. Following events are possible:<nl>
        FE_DATA_AVAIL<s2> 0x01 : New Data Received (in the data field)<nl>
        FE_READY_FOR_SEND<s2> 0x02 : (not supported at time)<nl>
        FE_ERROR_DETECTED<s2> 0x10 : (not supported at time)<nl><nl>
        Also the callback function could return a fossil_event_struct.
        Then the returned struct contains an action event which the caller of the callback
        (the fossil interface) has to do. Following events are possible:<nl>
        FE_IGNORE_DATA<s2> 0x01 : do not copy the reveived data in the Receive Queue<nl>
        FE_DATA_TO_SEND<s2> 0x02 : (not supported at time)<nl><nl>

        If the callback does not return any event, the return value must be a null pointer (=0L).
 @Also  fosA1.htm |  Fossil User Callback Function | | | |
*/
/******************************************************************************/
#ifdef _fosA1_
/*
 @doc DOC_fosA1_
 ---------------------------------------------------------------
 Install a Fossil User Callback Function
 ---------------------------------------------------------------
 @clib  fossil_user_callback | Install a Fossil User Callback Function.
        The User Callback function will be called when a fossil event occurs. |
        <cfont>void fossil_user_callback ( int port,<nl>
        <tab><tab><tab><tab><tab><tab><tab>FOSSIL_USER_CALLBACK pfFossUCB );<efont> |
 @parm_sc1x int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                <ule>
 @parm_sc1x3 int  | port  | Port specifier:<nl><ul>
                FOSSIL_EXT = 0 for EXT port<nl>
                FOSSIL_COM = 1 for COM port<nl>
                FOSSIL_SER2 = 2 for SER2 port<nl>
                FOSSIL_SER3 = 3 for SER3 port<nl>
                <ule>
 @parm  FOSSIL_USER_CALLBACK  | pfFossUCB  | Pointer to Fossil User CallBack function
 @comm  This call installs a User callback function for the specified serial port.
        The Callback function must be very short! Long fossil callback functions could be lead to characters losing.
        To use the fossil callback functions you have to switch the serial port into the IRQ Mode
        (use CHIP.INI to do that). In DMA Mode the callback functions does not work!<nl><nl>
        The callback function must be declared like the function below:<nl>
        <linkto>FosEvent.htm<linkmid>fossil_event_t<linktail>
        far *(huge my_fossil_callback)( <linkto>FosEvent.htm<linkmid>fossil_event_t<linktail> far *e )<nl><nl>
        The callback function must be installed like below:<nl>
        fossil_user_callback(PORT, (FOSSIL_USER_CALLBACK) my_fossil_callback);
 @Also  FosEvent.htm |  Fossil Event Data Structure | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../fossil.htm#0x140xA1<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.10 | V1.00 | V1.00 | V0.90
*/
void fossil_user_callback(int port, FOSSIL_USER_CALLBACK pfFossUCB)
{
    asm { LES  DI, pfFossUCB } ;   // ES:DI = far ptr to the User Callback function
    _DX = port;
    _AH = 0xA1 ;                   // Service index
    asm { int FOSSILVECT  } ;
}
#endif

/****************************************************************************/
// end of file
/****************************************************************************/

